import os
path = os.path
from random import randrange

from myhdl import (block, Signal, intbv, delay, instance, downrange)
from myhdl.conversion import verify, analyze
from myhdl import ConversionError
from myhdl.conversion._misc import _error


@block
def ForLoopError1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in (1, 2, 3):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoopError2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in list((1, 2, 3)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoopError3(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(1, 4, -1):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a), 5):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop3(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a), 3, 2):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop4(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(len(a)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop5(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(6, len(a)):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForLoop6(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in range(5, len(a), 3):
                if a[i] == 1:
                    var += 1
            out.next = var

    return comb


@block
def ForContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                var += 1
            out.next = var

    return comb


@block
def ForBreakLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = 0
            for i in downrange(len(a)):
                if a[i] == 1:
                    out.next = i
                    break

    return comb


@block
def ForBreakContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                out.next = i
                break

    return comb


@block
def NestedForLoop1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                else:
                    for j in downrange(i):
                        if a[j] == 0:
                            var += 1
                    break
            out.next = var

    return comb


@block
def NestedForLoop2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = 0
            for i in downrange(len(a)):
                if a[i] == 0:
                    continue
                else:
                    for j in downrange(i - 1):
                        if a[j] == 0:
                            pass
                        else:
                            out.next = j
                            break
                    break

    return comb


def ReturnFromFunction(a):
    for i in downrange(len(a)):
        if a[i] == 1:
            return i
    return 0


@block
def FunctionCall(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = ReturnFromFunction(a)

    return comb


# During the following check, I noticed that non-blocking assignments
# are not scheduled when a task is disabled in Icarus. Apparently
# this is one of the many vague areas in the Verilog standard.
def ReturnFromTask(a, out):
    for i in downrange(len(a)):
        if a[i] == 1:
            out[:] = i
            return
    out[:] = 23  # to notice it


@block
def TaskCall(a, out):

    @instance
    def comb():
        var = intbv(0)[8:]
        while 1:
            yield a
            ReturnFromTask(a, var)
            out.next = var

    return comb


@block
def WhileLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            i = len(a) - 1
            while i >= 0:
                if a[i] == 1:
                    var += 1
                i -= 1
            out.next = var

    return comb


@block
def WhileContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            var = 0
            i = len(a) - 1
            while i >= 0:
                if a[i] == 0:
                    i -= 1
                    continue
                var += 1
                i -= 1
            out.next = var

    return comb


@block
def WhileBreakLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            i = len(a) - 1
            out.next = 0
            while i >= 0:
                if a[i] == 1:
                    out.next = i
                    break
                i -= 1

    return comb


@block
def WhileBreakContinueLoop(a, out):

    @instance
    def comb():
        while 1:
            yield a
            i = len(a) - 1
            out.next = 0
            while i >= 0:
                if a[i] == 0:
                    i -= 1
                    continue
                out.next = i
                break

    return comb


@block
def LoopBench(LoopTest):

    a = Signal(intbv(-1)[16:])
    z = Signal(intbv(0)[16:])

    looptest_inst = LoopTest(a, z)
    data = tuple([randrange(2 ** min(i, 16)) for i in range(100)])

    @instance
    def stimulus():
        for i in range(100):
            a.next = data[i]
            yield delay(10)
            print(z)

    return stimulus, looptest_inst


def testForLoopError1():
    try:
        analyze(LoopBench(ForLoopError1))
    except ConversionError as e:
        assert e.kind == _error.Requirement
    else:
        assert False


def testForLoopError2():
    try:
        analyze(LoopBench(ForLoopError2))
    except ConversionError as e:
        assert e.kind == _error.Requirement
    else:
        assert False


def testForLoopError3():
    try:
        analyze(LoopBench(ForLoopError3))
    except ConversionError as e:
        assert e.kind == _error.Requirement
    else:
        assert False


def testForLoop1():
    assert verify(LoopBench(ForLoop1)) == 0


def testForLoop2():
    assert verify(LoopBench(ForLoop2)) == 0


def testForLoop4():
    assert verify(LoopBench(ForLoop4)) == 0


def testForLoop5():
    assert verify(LoopBench(ForLoop5)) == 0

# for loop 3 and 6 can't work in vhdl


def testForContinueLoop():
    assert verify(LoopBench(ForContinueLoop)) == 0


def testForBreakLoop():
    assert verify(LoopBench(ForBreakLoop)) == 0


def testForBreakContinueLoop():
    assert verify(LoopBench(ForBreakContinueLoop)) == 0


def testNestedForLoop1():
    assert verify(LoopBench(NestedForLoop1)) == 0


def testNestedForLoop2():
    assert verify(LoopBench(NestedForLoop2)) == 0


def testFunctionCall():
    assert verify(LoopBench(FunctionCall)) == 0

# # def testTaskCall(self):
# #     sim = self.bench(TaskCall)
# #     Simulation(sim).run()


def testWhileLoop():
    assert verify(LoopBench(WhileLoop)) == 0


def testWhileContinueLoop():
    assert verify(LoopBench(WhileContinueLoop)) == 0


def testWhileBreakLoop():
    assert verify(LoopBench(WhileBreakLoop)) == 0


def testWhileBreakContinueLoop():
    assert verify(LoopBench(WhileBreakContinueLoop)) == 0



from myhdl import (block, Signal, intbv, instance, delay)


@block
def issue_133():
    z = Signal(False)
    large_signal = Signal(intbv(123456789123456, min=0, max=2 ** 256))

    @instance
    def check():
        z.next = large_signal[10]
        yield delay(10)
        print (large_signal[31:])
        print (large_signal[62:31])
        print (large_signal[93:62])

    return check


def test_issue_133():
    issue_133().verify_convert() == 0



import myhdl
from myhdl import *

from glibc_random import glibc_random


def max2(z, a, b):

    @always_comb
    def comb():
        if a > b:
            z.next = a
        else:
            z.next = b

    return comb


def maxn(z, a):
    L = len(a)
    assert L
    assert L % 2 == 0
    H = L // 2
    W = len(a[0])
    if L == 2:
        comp2 = max2(z, a[1], a[0])
        return comp2
    else:
        zlo = Signal(intbv(0)[W:])
        zhi = Signal(intbv(0)[W:])
        complo = maxn(zlo, a[H:])
        comphi = maxn(zhi, a[:H])
        comp2 = max2(z, zhi, zlo)
        return comp2, complo, comphi


def test_findmax():

    L = 32
    W = 16

    random_word = Signal(intbv(0)[L * W:])
    clock = Signal(bool())
    stopped = Signal(bool())
    a = [Signal(intbv(0)[W:]) for i in range(L)]
    z = Signal(intbv(0)[W:])

    dut = maxn(z, a)

    @instance
    def clockgen():
        clock.next = 0
        yield delay(10)
        while not stopped:
            clock.next = not clock
            yield delay(10)

    @instance
    def stimulus():
        stopped.next = 0
        yield delay(10)
        exp = intbv(0)[W:]
        val = intbv(0)[W:]
        random_word = intbv(0)[32:]
        random_word[:] = 93
        for i in range(2 ** 18):
            exp[:] = 0
            for s in range(L):
                random_word[:] = glibc_random(random_word)
                val[:] = random_word[W:]
                if exp < val:
                    exp[:] = val
                a[s].next = val
            yield clock.negedge
            assert z == exp
        stopped.next = 1
        yield delay(10)

    return dut, clockgen, stimulus


if __name__ == '__main__':
    sim = Simulation(test_findmax())
    sim.run()





from myhdl import Signal, always_seq, ResetSignal
from myhdl._always_seq import AlwaysSeqError, _error
from helpers import raises_kind


def test_clock():
    """ check the edge parameter """

    # should fail without a valid Signal
    clock = Signal(bool(0))
    reset = ResetSignal(0, active=0, isasync=True)

    with raises_kind(AlwaysSeqError, _error.EdgeType):

        @always_seq(clock, reset=reset)
        def logic1():
            pass

    # should work with a valid Signal
    clock = Signal(bool(0))
    try:

        @always_seq(clock.posedge, reset=reset)
        def logic2():
            pass

    except:
        assert False


def test_reset():
    """ check the reset parameter """

    # should fail without a valid ResetSignal
    clock = Signal(bool(0))
    reset = Signal(bool(0))

    with raises_kind(AlwaysSeqError, _error.ResetType):

        @always_seq(clock.posedge, reset=reset)
        def synch():
            pass

    # should work with a valid Signal
    reset = ResetSignal(0, active=0, isasync=True)
    try:

        @always_seq(clock.posedge, reset=reset)
        def synch2():
            pass

    except:
        assert False



import os
path = os.path
import unittest
from unittest import TestCase

from myhdl import (block, Signal, intbv, delay, always_comb, instance)
from myhdl._Simulation import Simulation

from .util import setupCosimulation


@block
def bin2gray2(B, G, width):
    """ Gray encoder.

    B -- input intbv signal, binary encoded
    G -- output intbv signal, gray encoded
    width -- bit width
    """

    @instance
    def comb():
        Bext = intbv(0)[width + 1:]
        while 1:
            yield B
            Bext[:] = B
            for i in range(width):
                G.next[i] = Bext[i + 1] ^ Bext[i]

    return comb


@block
def bin2gray(B, G, width):

    """ Gray encoder.

    B -- input intbv signal, binary encoded
    G -- output intbv signal, gray encoded
    width -- bit width
    
    """

    @always_comb
    def comb():
        Bext = intbv(0)[width + 1:]
        Bext[:] = B
        for i in range(width):
            G.next[i] = Bext[i + 1] ^ Bext[i]

    return comb


objfile = "bin2gray.o"
analyze_cmd = "iverilog -o %s bin2gray_inst.v tb_bin2gray_inst.v" % objfile
simulate_cmd = "vvp -m ../../../cosimulation/icarus/myhdl.vpi %s" % objfile

# @block
# def bin2gray_v(B, G):
#     if path.exists(objfile):
#         os.remove(objfile)
#     os.system(analyze_cmd)
#     return Cosimulation(simulate_cmd, **locals())


@block
def bin2gray_v(name, B, G):
    return setupCosimulation(**locals())


class TestBin2Gray(TestCase):

    def bench(self, width, bin2gray):

        B = Signal(intbv(0)[width:])
        G = Signal(intbv(0)[width:])
        G_v = Signal(intbv(0)[width:])

        bin2gray_inst = bin2gray(B, G, width).convert(hdl='Verilog')
        # bin2gray_inst = bin2gray(B, G, width)
        bin2gray_v_inst = bin2gray_v(bin2gray.__name__, B, G_v)

        def stimulus():
            for i in range(2 ** width):
                B.next = intbv(i)
                yield delay(10)
                # print "B: " + bin(B, width) + "| G_v: " + bin(G_v, width)
                # print bin(G, width)
                # print bin(G_v, width)
                self.assertEqual(G, G_v)

        return bin2gray_v_inst, stimulus(), bin2gray_inst

    def test1(self):
        sim = self.bench(width=8, bin2gray=bin2gray)
        Simulation(sim).run()

    def test2(self):
        sim = self.bench(width=8, bin2gray=bin2gray2)
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()




from myhdl import (block, Signal, intbv, always_comb, always)
from myhdl import ConversionError
from myhdl.conversion._misc import _error


class HdlObj(object):

    def __init__(self):
        pass

    @block
    def method_func(self, clk, srst, x, y):
        z = Signal(intbv(0, min=y.min, max=y.max))
        ifx = self._mfunc(x, z)

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + z  # x + (x+1)

        return hdl, ifx

    @block
    def _mfunc(self, x, y):

        @always_comb
        def _hdl():
            y.next = x + 1

        return _hdl


@block
def _func(x, y):

    @always_comb
    def _hdl():
        y.next = x + 1

    return _hdl


class HdlObjObj(object):

    def __init__(self):
        pass

    @block
    def method_func(self, clk, srst, x, y):
        z1 = Signal(intbv(0, min=y.min, max=y.max))
        z2 = Signal(intbv(0, min=y.min, max=y.max))
        hobj = HdlObj()
        ifx1 = hobj._mfunc(x, z1)
        ifx2 = _func(x, z2)

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + z1 + (z1 - z2)

        return hdl, ifx1, ifx2


class HdlObjAttrSimple(object):

    def __init__(self):
        self.Constant = 3

    @block
    def method_func(self, clk, srst, x, y):

        # limitation for class method conversion, the object attributes
        # can only be used/accessed during elaboration.
        Constant = int(self.Constant)

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + (x + 1) + Constant - 3

        return hdl


class HdlObjNotSelf(object):
    ''' testing against using 'this' iso 'self '''

    def __init__(this):
        pass

    @block
    def method_func(this, clk, srst, x, y):

        @always(clk.posedge)
        def hdl():
            if srst:
                y.next = 0
            else:
                y.next = x + 1

        return hdl


def test_hdlobj():
    clk = Signal(False)
    srst = Signal(False)
    x = Signal(intbv(0, min=0, max=16))
    y = Signal(intbv(0, min=0, max=16))
    hdlobj_inst = HdlObj()
    hdlobj_inst.method_func(clk, srst, x, y).analyze_convert()


def test_hdlobjobj():
    clk = Signal(False)
    srst = Signal(False)
    x = Signal(intbv(0, min=0, max=16))
    y = Signal(intbv(0, min=0, max=16))
    hdlobj_inst = HdlObjObj()
    hdlobj_inst.method_func(clk, srst, x, y).analyze_convert()


def test_hdlobjattrsimple():
    clk = Signal(False)
    srst = Signal(False)
    x = Signal(intbv(0, min=0, max=16))
    y = Signal(intbv(0, min=0, max=16))
    hdlobj_inst = HdlObjAttrSimple()
    hdlobj_inst.method_func(clk, x, srst, y).analyze_convert()


def test_hdlobjnotself():
    clk = Signal(False)
    srst = Signal(False)
    x = Signal(intbv(0, min=0, max=16))
    y = Signal(intbv(0, min=0, max=16))
    hdlobj_inst = HdlObjNotSelf()
    try:
        hdlobj_inst.method_func(clk, x, srst, y).analyze_convert()
    except ConversionError as e:
        assert e.kind == _error.NotSupported
    else:
        assert False



import os
path = os.path
import unittest
from random import randrange

from myhdl import (block, Signal, intbv, delay, always,
                    StopSimulation, negedge)
from myhdl._Simulation import Simulation

from .test_bin2gray import bin2gray
from .test_inc import inc

from .util import setupCosimulation

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def GrayInc_0(graycnt, enable, clock, reset, width):

    bincnt = Signal(intbv(0)[width:])

    inc_1 = inc(bincnt, enable, clock, reset, n=2 ** width)
    bin2gray_1 = bin2gray(B=bincnt, G=graycnt, width=width)

    return inc_1, bin2gray_1


@block
def GrayIncReg_0(graycnt, enable, clock, reset, width):

    graycnt_comb = Signal(intbv(0)[width:])

    gray_inc_1 = GrayInc_0(graycnt_comb, enable, clock, reset, width)

    @always(clock.posedge)
    def reg_1():
        graycnt.next = graycnt_comb

    return gray_inc_1, reg_1


width = 8
graycnt = Signal(intbv(0)[width:])
enable, clock, reset = [Signal(bool()) for i in range(3)]
# GrayIncReg(graycnt, enable, clock, reset, width)


def GrayIncReg_v(name, graycnt, enable, clock, reset, width):
    return setupCosimulation(**locals())


graycnt_v = Signal(intbv(0)[width:])


class TestGrayInc(unittest.TestCase):

    def clockGen(self):
        while 1:
            yield delay(10)
            clock.next = not clock

    def stimulus(self):
        reset.next = ACTIVE_LOW
        yield negedge(clock)
        reset.next = INACTIVE_HIGH
        for i in range(1000):
            enable.next = 1
            yield clock.negedge
        for i in range(1000):
            enable.next = min(1, randrange(5))
            yield clock.negedge
        raise StopSimulation

    def check(self):
        yield reset.posedge
        self.assertEqual(graycnt, graycnt_v)
        while 1:
            yield clock.posedge
            yield delay(1)
            # print "%d graycnt %s %s" % (now(), graycnt, graycnt_v)
            self.assertEqual(graycnt, graycnt_v)

    def bench(self):
        gray_inc_reg_1 = GrayIncReg_0(graycnt, enable, clock, reset, width).convert(hdl='Verilog')
        gray_inc_reg_v = GrayIncReg_v(GrayIncReg_0.__name__, graycnt_v, enable, clock, reset, width)
        clk_1 = self.clockGen()
        st_1 = self.stimulus()
        ch_1 = self.check()
        sim = Simulation(gray_inc_reg_1, gray_inc_reg_v, clk_1, st_1, ch_1)
        return sim

    def test_grayinc(self):
        """ Check gray inc operation """
        sim = self.bench()
        sim.run(quiet=1)




import myhdl
from myhdl import *


def lfsr24(lfsr, enable, clock, reset):

    @always(clock.posedge, reset.posedge)
    def comb():
        if reset == 1:
            lfsr.next = 1
        else:
            if enable:
                # lfsr.next[24:1] = lfsr[23:0]
                lfsr.next = lfsr << 1
                lfsr.next[0] = lfsr[23] ^ lfsr[22] ^ lfsr[21] ^ lfsr[16]

    return comb




import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, delay, always,
                   instance, StopSimulation)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def edge1(flag, sig, clock):

    sig_Z1 = Signal(bool(0))

    @always(clock.posedge)
    def detect():
        sig_Z1.next = sig
        flag.next = False
        if sig and not sig_Z1:
            flag.next = True

    return detect


@block
def edge2(flag, sig, clock):

    sig_Z1 = Signal(bool(0))

    @always(clock.posedge)
    def detect():
        sig_Z1.next = sig
        flag.next = sig and not sig_Z1

    return detect


@block
def edge3(flag, sig, clock):

    @instance
    def detect():
        sig_Z1 = False
        while True:
            yield clock.posedge
            flag.next = sig and not sig_Z1
            sig_Z1 = sig.val

    return detect


@block
def edge4(flag, sig, clock):

    @instance
    def detect():
        sig_Z1 = False
        while True:
            yield clock.posedge
            flag.next = sig and not sig_Z1
            sig_Z1 = bool(sig)

    return detect


@block
def edge_v(name, flag, sig, clock):
    return setupCosimulation(**locals())


class TestEdge(TestCase):

    def bench(self, edge):

        clock = Signal(bool(0))
        sig = Signal(bool(0))
        flag = Signal(bool(0))

        sig_Z1 = Signal(bool(0))
        sig_Z2 = Signal(bool(0))

        @always(delay(10))
        def clockgen():
            clock.next = not clock

        @instance
        def stimulus():
            yield clock.negedge
            for dummy in range(100):
                sig.next = randrange(2)
                yield clock.negedge
            raise StopSimulation

        @always(clock.posedge)
        def delayline():
            sig_Z1.next = sig
            sig_Z2.next = sig_Z1

        @always(clock.negedge)
        def check():
            expected = sig_Z1 and not sig_Z2
            self.assertEqual(flag, expected)

        edge_inst = edge(flag, sig, clock).convert(hdl='Verilog')
        edge_inst_v = edge_v(edge.__name__, flag, sig, clock)

        return clockgen, stimulus, delayline, check, edge_inst_v

    def testEdge1(self):
        sim = Simulation(self.bench(edge1))
        sim.run(quiet=1)

    def testEdge2(self):
        sim = Simulation(self.bench(edge2))
        sim.run(quiet=1)

    def testEdge3(self):
        sim = Simulation(self.bench(edge3))
        sim.run(quiet=1)

    def testEdge4(self):
        sim = Simulation(self.bench(edge4))
        sim.run(quiet=1)


if __name__ == '__main__':
    unittest.main()




'''
Created on 23 aug. 2018

@author: josy
'''

from myhdl import Signal, intbv, block, always_comb, always_seq, ResetSignal, instance, \
                    delay, StopSimulation

# making up something sensible or even useful


class EncoderUD(object):

    def __init__(self):
        self.up = Signal(bool(0))
        self.down = Signal(bool(0))
        self.zero = Signal(bool(0))


class TableXY(object):

    def __init__(self):
        self.x = EncoderUD()
        self.y = EncoderUD()


class Cartesian2D(object):

    def __init__(self):
        self.x = Signal(intbv(0, -2 ** 15, 2 ** 15))
        self.y = Signal(intbv(0, -2 ** 15, 2 ** 15))


@block
def Encoder(Clk, Reset, Encoder, Position):
    lpos = Signal(intbv(0, -2 ** 15, 2 ** 15))

    @always_seq(Clk.posedge, reset=Reset)
    def synch():
        if Encoder.zero:
            lpos.next = 0
        elif Encoder.up and not Encoder.down:
            if lpos < 2 ** 15:
                lpos.next = lpos + 1
        elif not Encoder.up and Encoder.down:
            if lpos > -2 ** 15:
                lpos.next = lpos - 1

    @always_comb
    def comb():
        Position.next = lpos

    return synch, comb


@block
def XYTable(Clk, Reset, Table, Position):
    '''
        Encoding the X,Y position of an XY Table using two Encoders
        Clk, Reset: as usual
        Table: an TableXY() object giving us the input
        Position: an Cartesian2D() object telling us the position    
    '''
    pos = Cartesian2D()
    tablex = Encoder(Clk, Reset, Table.x, pos.x)
    tablex.name = 'Table_X'
    tabley = Encoder(Clk, Reset, Table.y, pos.y)
    tabley.name = 'Table_Y'

    @always_comb
    def assign():
        Position.x.next = pos.x
        Position.y.next = pos.y

    return tablex, tabley, assign


def test_top_level_interfaces_analyze():
    Clk = Signal(bool(0))
    Reset = ResetSignal(0, 1, True)
    Table = TableXY()
    Position = Cartesian2D()

    dfc = XYTable(Clk, Reset, Table, Position)
    assert dfc.analyze_convert() == 0


@block
def tb_top_level_interfaces():

    Clk = Signal(bool(0))
    Reset = ResetSignal(0, 1, False)
    Table = TableXY()
    Position = Cartesian2D()

    tb_dut = XYTable(Clk, Reset, Table, Position)

    xul = tuple([0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1])
    xdl = tuple([0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1])
    yul = tuple([0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1])
    ydl = tuple([0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1])

    tCK = 20
    tReset = int(tCK * 3.5)

    @instance
    def tb_clk():
        Clk.next = False
        yield delay(int(tCK // 2))
        while True:
            Clk.next = not Clk
            yield delay(int(tCK // 2))

    @instance
    def tb_stim():
        Table.x.up.next = 0
        Table.x.down.next = 0
        Table.y.up.next = 0
        Table.y.down.next = 0
        Reset.next = 1
        yield delay(tReset)
        Reset.next = 0
        yield Clk.negedge

        for i in range(len(xul)):
            yield Clk.negedge
            Table.x.up.next = xul[i]
            Table.x.down.next = xdl[i]
            Table.y.up.next = yul[i]
            Table.y.down.next = ydl[i]
            yield Clk.posedge
            print("%d: %d %d" % (i, Position.x, Position.y))

        yield Clk.posedge
        Table.x.zero.next = 1
        Table.y.zero.next = 1
        yield Clk.posedge
        Table.x.zero.next = 0
        Table.y.zero.next = 0
        yield Clk.posedge
        assert Position.x == 0
        assert Position.y == 0

        raise StopSimulation

    return tb_dut, tb_clk, tb_stim


def test_top_level_interfaces_verify():
    inst = tb_top_level_interfaces()
    assert inst.verify_convert() == 0


if __name__ == '__main__':
    Clk = Signal(bool(0))
    Reset = ResetSignal(0, 1, True)
    Table = TableXY()
    Position = Cartesian2D()

    dft = tb_top_level_interfaces()
    dft.run_sim()

    dfc = XYTable(Clk, Reset, Table, Position)
    dfc.name = 'XYTable'
    dfc.convert('Verilog')
    dfc.convert('VHDL')



#!/usr/bin/env python
#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the intbv.signed() unit tests. """
from myhdl import (block, intbv, delay, concat, instance, conversion)


@block
def PlainIntbv():
    '''Test a plain intbv instance with .signed() 

    ----+----+----+----+----+----+----+----
       -3   -2   -1    0    1    2    3

                      min  max
                           min  max
                 min       max
                 min            max
            min            max
            min       max
            min  max
          neither min nor max is set
          only max is set
          only min is set

    '''
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # in the following cases the .signed() function should classify the
    # value of the intbv instance as unsigned and return the 2's
    # complement value of the bits as specified by _nrbits.
    #

    @instance
    def comb():

        print("Plain Instance Test")

        yield delay(10)
        # intbv with positive range, pos number, and msb not set, return signed()
        # Expect the number to be returned
        a1 = intbv(0x3b, min=0, max=0x7c)
        b1 = a1.signed()
        assert b1 == 0x3b

        # intbv with positive range, pos number, and msb set, return signed()
        # test various bit patterns to see that the 2's complement
        # conversion works correct
        # Expect the number to be converted to a negative number
        a2 = intbv(7, min=0, max=8)
        b2 = a2.signed()
        assert b2 == -1

        a3 = intbv(6, min=0, max=8)
        b3 = a3.signed()
        assert b3 == -2

        a4 = intbv(5, min=0, max=8)
        b4 = a4.signed()
        assert b4 == -3

        # set bit #3 and increase the range so that the set bit is considered
        # the sign bit. Here min = 0
        # Expect to return -4
        a5 = intbv(4, min=0, max=5)
        b5 = a5.signed()
        assert b5 == -4

        a6 = intbv(4, min=0, max=6)
        b6 = a6.signed()
        assert b6 == -4

        a7 = intbv(4, min=0, max=7)
        b7 = a7.signed()
        assert b7 == -4

        a8 = intbv(4, min=0, max=8)
        b8 = a8.signed()
        assert b8 == -4

        # here it is not the sign bit anymore
        # Expect the value to be 4
        a9 = intbv(4, min=0, max=9)
        b9 = a9.signed()
        assert b9 == 4

        # set bit #3 and increase the range so that the set bit is considered
        # the sign bit. Here min > 0
        # Expect to return -4
        a10 = intbv(4, min=1, max=5)
        b10 = a10.signed()
        assert b10 == -4

        a11 = intbv(4, min=2, max=6)
        b11 = a11.signed()
        assert b11 == -4

        a12 = intbv(4, min=3, max=7)
        b12 = a12.signed()
        assert b12 == -4

        a13 = intbv(4, min=4, max=8)
        b13 = a13.signed()
        assert b13 == -4

        # again with min > 0, here it is not the sign bit anymore
        # Expect the value to be 4
        a14 = intbv(4, min=2, max=9)
        b14 = a14.signed()
        assert b14 == 4

        # intbv with positive range, value = 0, return signed()
        # Expect the number to be returned
        a15 = intbv(0, min=0, max=0x8)
        b15 = a15.signed()
        assert b15 == 0

        #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        # in these cases the .signed() function should classify the
        # value of the intbv instance as signed and return the value as is
        #

        # set bit #3 and increase the range that the set bit is actually the
        # msb, but due to the negative range not considered signed
        # Expect to return 4
        a20 = intbv(4, min=-1, max=5)
        b20 = a20.signed()
        assert b20 == 4

        a21 = intbv(4, min=-1, max=6)
        b21 = a21.signed()
        assert b21 == 4

        a22 = intbv(4, min=-1, max=7)
        b22 = a22.signed()
        assert b22 == 4

        a23 = intbv(4, min=-1, max=8)
        b23 = a23.signed()
        assert b23 == 4

        # intbv with negative range, pos number, and msb set, return signed()
        # Expect the number to returned as is
        a24 = intbv(7, min=-1, max=8)
        b24 = a24.signed()
        assert b24 == 7

        a25 = intbv(6, min=-1, max=8)
        b25 = a25.signed()
        assert b25 == 6

        a26 = intbv(5, min=-1, max=8)
        b26 = a26.signed()
        assert b26 == 5

        # intbv with symmetric (min = -max) range, pos value, msb set
        # return signed()
        # Expect value returned as is
        a27 = intbv(4, min=-8, max=8)
        b27 = a27.signed()
        assert b27 == 4

        # intbv with symmetric (min = -max) range, neg value,
        # return signed()
        # Expect value returned as is
        a28 = intbv(-4, min=-8, max=8)
        b28 = a28.signed()
        assert b28 == -4

        # intbv with symmetric (min=-max) range, value = 0,
        # return signed()
        # Expect value returned as is
        a29 = intbv(0, min=-8, max=8)
        b29 = a29.signed()
        assert b29 == 0

    return comb


@block
def SlicedSigned():
    '''Test a slice with .signed()

    This test can actually be simplified, as a slice will always have
    min=0 and max > min, which will result in an intbv instance that
    will be considered unsigned by the intbv.signed() function.
    '''

    @instance
    def comb():
        b = intbv(4, min=-8, max=8)
        a = intbv(4, min=-8, max=8)
        print("SLicedSigned test")
        yield delay(10)
        b[:] = a[4:]
        assert b == 4
        b[:] = a[4:].signed()
        assert b == 4  # msb is not set with a 4 bit slice

        b[:] = a[3:]
        assert b == 4
        b[:] = a[3:].signed()
        assert b == -4  # msb is set with 3 bits sliced

    return comb


@block
def SignedConcat():
    '''Test the .signed() function with the concatenate function'''

    @instance
    def comb():
        print("Signed Concat test")
        yield delay(10)

        # concat 3 bits
        # Expect the signed function to return a negative value
        a = intbv(0)[3:]
        a[:] = concat(True, True, True)
        assert a == 7
        assert a.signed() == -1
        assert concat(True, True, True).signed() == -1

        # concat a 3 bit intbv with msb set and two bits
        # Expect a negative number
        b = intbv(5, min=0, max=8)
        assert concat(b, True, True).signed() == -9

    return comb


def test_PlainIntbv():
    assert conversion.verify(PlainIntbv()) == 0


def test_SlicedSigned():
    assert conversion.verify(SlicedSigned()) == 0


def test_SignedConcat():
    assert conversion.verify(SignedConcat()) == 0




import myhdl
from myhdl import *


def inc_comb(nextCount, count, n):

    @always(count)
    def comb():
        # do nothing here
        pass

    nextCount.driven = "wire"

    return comb


inc_comb.verilog_code = \
"""
assign $nextCount = ($count + 1) % $n;
"""

inc_comb.vhdl_code = \
"""
$nextCount <= ($count + 1) mod $n;
"""


def main():
    m = 8
    n = 2 ** m
    count = Signal(intbv(0)[m:])
    nextCount = Signal(intbv(0)[m:])
    toVerilog(inc_comb, nextCount, count, n)
    toVHDL(inc_comb, nextCount, count, n)


if __name__ == '__main__':
    main()




import os
path = os.path
import unittest
from unittest import TestCase
import random
from random import randrange
random.seed(2)

from myhdl import (block, Signal, intbv, delay, always,
                   instance, StopSimulation)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

ACTIVE_LOW, INACTIVE_HIGH = 0, 1


@block
def decRef(count, enable, clock, reset, n):
    """ Decrementer with enable.
    
    count -- output
    enable -- control input, decrement when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def comb():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    if count == -n:
                        count.next = n - 1
                    else:
                        count.next = count - 1

    return comb


@block
def dec(count, enable, clock, reset, n):
    """ Decrementer with enable.
    
    count -- output
    enable -- control input, decrement when 1
    clock -- clock input
    reset -- asynchronous reset input
    n -- counter max value
    """

    @instance
    def decProcess():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                if enable:
                    if count == -n:
                        count.next = n - 1
                    else:
                        count.next = count - 1

    return decProcess


@block
def decFunc(count, enable, clock, reset, n):

    def decFuncFunc(cnt):
        count_next = intbv(0, min=-n, max=n)
        if count == -n:
            count_next[:] = n - 1
        else:
            count_next[:] = cnt - 1
        return count_next

    @always(clock.posedge, reset.negedge)
    def decFuncGen():
        if reset == ACTIVE_LOW:
            count.next = 0
        else:
            if enable:
                count.next = decFuncFunc(count)

    return decFuncGen


@block
def decTask(count, enable, clock, reset, n):

    def decTaskFunc(cnt, enable, reset, n):
        if enable:
            if cnt == -n:
                cnt.next = n - 1
            else:
                cnt.next = cnt - 1

    @instance
    def decTaskGen():
        cnt = intbv(0, min=-n, max=n)
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                cnt[:] = 0
                count.next = 0
            else:
                # print count
                decTaskFunc(cnt, enable, reset, n)
                count.next = cnt

    return decTaskGen


@block
def decTaskFreeVar(count, enable, clock, reset, n):

    def decTaskFunc():
        if enable:
            if count == -n:
                count.next = n - 1
            else:
                count.next = count - 1

    @instance
    def decTaskGen():
        while 1:
            yield clock.posedge, reset.negedge
            if reset == ACTIVE_LOW:
                count.next = 0
            else:
                # print count
                decTaskFunc()

    return decTaskGen


def dec_v(name, count, enable, clock, reset):
    return setupCosimulation(**locals())


class TestDec(TestCase):

    def clockGen(self, clock):
        while 1:
            yield delay(10)
            clock.next = not clock

    def stimulus(self, enable, clock, reset):
        reset.next = INACTIVE_HIGH
        yield clock.negedge
        reset.next = ACTIVE_LOW
        yield clock.negedge
        reset.next = INACTIVE_HIGH
        for dummy in range(1000):
            enable.next = 1
            yield clock.negedge
        for dummy in range(1000):
            enable.next = min(1, randrange(5))
            yield clock.negedge
        raise StopSimulation

    def check(self, count, count_v, enable, clock, reset, n):
        expect = 0
        yield reset.posedge
        self.assertEqual(count, expect)
        self.assertEqual(count, count_v)
        while 1:
            yield clock.posedge
            if enable:
                if expect == -n:
                    expect = n - 1
                else:
                    expect -= 1
            yield delay(1)
            # print "%d count %s expect %s count_v %s" % (now(), count, expect, count_v)
            self.assertEqual(count, expect)
            self.assertEqual(count, count_v)

    def bench(self, dec):

        m = 8
        n = 2 ** (m - 1)

        count = Signal(intbv(0, min=-n, max=n))
        count_v = Signal(intbv(0, min=-n, max=n))
        enable = Signal(bool(0))
        clock, reset = [Signal(bool()) for __ in range(2)]

        dec_inst_ref = decRef(count, enable, clock, reset, n=n)
        dec_inst = dec(count, enable, clock, reset, n=n).convert(hdl='Verilog')
        # dec_inst = dec(count, enable, clock, reset, n=n)
        dec_inst_v = dec_v(dec.__name__, count_v, enable, clock, reset)
        clk_1 = self.clockGen(clock)
        st_1 = self.stimulus(enable, clock, reset)
        ch_1 = self.check(count, count_v, enable, clock, reset, n=n)

        sim = Simulation(dec_inst_ref, dec_inst_v, clk_1, st_1, ch_1)
        return sim

    def testDecRef(self):
        sim = self.bench(decRef)
        sim.run(quiet=1)

    def testDec(self):
        sim = self.bench(dec)
        sim.run(quiet=1)

    def testDecFunc(self):
        sim = self.bench(decFunc)
        sim.run(quiet=1)

# # signed inout in task doesn't work yet in Icarus
# #     def testDecTask(self):
# #         sim = self.bench(decTask)
# #         sim.run(quiet=1)

    def testDecTaskFreeVar(self):
        sim = self.bench(decTaskFreeVar)
        sim.run(quiet=0)


if __name__ == '__main__':
    unittest.main()




from myhdl import (block, Signal, intbv, always_comb)


@block
def bug_28(dout, channel):

    @always_comb
    def comb():
        dout.next = 0x8030 + (channel << 10)

    return comb


dout = Signal(intbv(0)[16:0])
channel = Signal(intbv(0)[4:0])


def test_bug_28():
    try:
        bug_28(dout, channel).convert(hdl='VHDL')
    except:
        raise



import os
path = os.path
import unittest

from myhdl import (block, Signal, enum, intbv, delay, instance,
                   StopSimulation, toVerilog)
from myhdl._Simulation import Simulation
from myhdl import ConversionError
from myhdl.conversion._misc import _error

from .util import setupCosimulation

b = c = 2


@block
def UnboundError1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = a + b
            b = 1

    return comb


@block
def UnboundError2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            if a == 1:
                c = 1
            else:
                out.next = c

    return comb


@block
def UnboundError3(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = a + d  # this is the error we want to test
            d = 1

    return comb


@block
def UnboundError4(a, out):

    @instance
    def comb():
        while 1:
            yield a
            if a == 1:
                e = 1
            else:
                out.next = e

    return comb


@block
def InferError1(a, out):

    @instance
    def comb():
        yield a
        b = 2
        b = intbv(0)[5:]
        b[:] = 4
        out.next = b

    return comb


@block
def InferError2(a, out):

    @instance
    def comb():
        yield a
        c = intbv(0)[5:]
        c[:] = 4
        c = intbv(0)[4:]
        c[:] = 4
        out.next = c

    return comb


enumType = enum("a", "b", "c")


@block
def InferError3(a, out):

    @instance
    def comb():
        yield a
        d = enumType.a
        d = 4
        out.next = b

    return comb


@block
def InferError4(a, out):

    @instance
    def comb():
        h = intbv(0)
        yield a
        out.next = h

    return comb


def InferError5Func(a):
    h = intbv(0)[5:]
    if a:
        return h
    else:
        return 1


@block
def InferError5(a, out):

    @instance
    def comb():
        yield a
        out.next = InferError5Func(a)

    return comb


def InferError6Func(a):
    if a:
        return intbv(0)
    else:
        return intbv(1)


@block
def InferError6(a, out):

    @instance
    def comb():
        yield a
        out.next = InferError6Func(a)

    return comb


def InferError7Func(a):
    if a:
        return intbv(0)[5:]
    else:
        return intbv(0xff)[7:2]


@block
def InferError7(a, out):

    @instance
    def comb():
        yield a
        out.next = InferError7Func(a)

    return comb


class TestErrors(unittest.TestCase):

    def check(self, *args):
        try:
            i = toVerilog(*args)
        except ConversionError as e:
            self.assertEqual(e.kind, _error.NotSupported)
        except:
            self.fail()
        else:
            self.fail()

    def check(self, Infertest, err):
        a = Signal(intbv(-1)[16:])
        out_v = Signal(intbv(0)[16:])
        out = Signal(intbv(0)[16:])
        try:
            infertest_inst = Infertest(a, out).convert(hdl='Verilog')
        except ConversionError as e:
            self.assertEqual(e.kind, err)
        except:
            self.fail()
        else:
            self.fail()

    def nocheck(self, Infertest, err=None):
        a = Signal(intbv(-1)[16:])
        out_v = Signal(intbv(0)[16:])
        out = Signal(intbv(0)[16:])
        infertest_inst = Infertest(a, out).convert(hdl='Verilog')

    def testUnboundError1(self):
        sim = self.check(UnboundError1, _error.UnboundLocal)

    def testUnboundError2(self):
        sim = self.check(UnboundError2, _error.UnboundLocal)

    def testUnboundError3(self):
        sim = self.check(UnboundError3, _error.UnboundLocal)

    def testUnboundError4(self):
        sim = self.check(UnboundError4, _error.UnboundLocal)

    def testInferError1(self):
        sim = self.check(InferError1, _error.TypeMismatch)

    def testInferError2(self):
        sim = self.check(InferError2, _error.NrBitsMismatch)

    def testInferError3(self):
        sim = self.check(InferError3, _error.TypeMismatch)

    def testInferError4(self):
        sim = self.check(InferError4, _error.IntbvBitWidth)

    def testInferError5(self):
        sim = self.check(InferError5, _error.ReturnTypeMismatch)

    def testInferError6(self):
        sim = self.check(InferError6, _error.ReturnIntbvBitWidth)

    def testInferError7(self):
        sim = self.nocheck(InferError7, _error.ReturnIntbvBitWidth)


@block
def Infer1(a, out):

    @instance
    def comb():
        while 1:
            yield a
            c = 5
            c = a < 4
            c = bool(0)
            c = False
            c = not a
            c = True
            out.next = c

    return comb


@block
def Infer2(a, out):

    @instance
    def comb():
        while 1:
            yield a
            c = a < 4
            c = bool(0)
            c = False
            c = not a
            c = True
            c = 5
            out.next = c

    return comb


def Infer3Func(a):
    if True:
        return a > 0
    else:
        return 5


@block
def Infer3(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = Infer3Func(a)

    return comb


def Infer4Func(a):
    while 1:
        if True:
            return 6
        else:
            return a < 3


@block
def Infer4(a, out):

    @instance
    def comb():
        while 1:
            yield a
            out.next = Infer4Func(a)

    return comb


@block
def Infer5(a, out):

    @instance
    def comb():
        while 1:
            yield a
            c = a + 1
            c = a - 1
            c = a * 3
            c = a // 2
            c = a << 2
            c = a >> 2
            c = a % 16
            c = +a
            c = -(-a)
            c = ~(-3)
            c = not a
            c = 5 & 4
            c = 5 | 2
            c = 6 ^ 3
            c = bool(a) and 1
            out.next = c

    return comb


def Infertest_v(name, a, out):
    return setupCosimulation(**locals())


class TestInfer(unittest.TestCase):

    def bench(self, Infertest):

        a = Signal(intbv()[16:])
        out_v = Signal(intbv(0)[16:])
        out = Signal(intbv(0)[16:])

        infertest_inst = Infertest(a, out).convert(hdl='Verilog')
        # infertest_inst = Infertest(hec, header)
        infertest_v_inst = Infertest_v(Infertest.__name__, a, out_v)

        def stimulus():
            a.next = 1
            yield delay(10)
            # print "%s %s" % (out, out_v)
            self.assertEqual(out, out_v)
            raise StopSimulation

        return stimulus(), infertest_inst, infertest_v_inst

    def testInfer1(self):
        sim = self.bench(Infer1)
        Simulation(sim).run()

    def testInfer2(self):
        sim = self.bench(Infer2)
        Simulation(sim).run()

    def testInfer3(self):
        sim = self.bench(Infer3)
        Simulation(sim).run()

    def testInfer4(self):
        sim = self.bench(Infer4)
        Simulation(sim).run()

    def testInfer5(self):
        sim = self.bench(Infer5)
        Simulation(sim).run()


if __name__ == '__main__':
    unittest.main()



import ast
import itertools
from types import FunctionType

from myhdl._util import _flatten
from myhdl._enum import EnumType
from myhdl._Signal import SignalType


class Data():
    pass


def _resolveRefs(symdict, arg):
    gens = _flatten(arg)
    data = Data()
    data.symdict = symdict
    v = _AttrRefTransformer(data)
    for gen in gens:
        v.visit(gen.ast)
    return data.objlist

# TODO: Refactor this into two separate nodetransformers, since _resolveRefs
# needs only the names, not the objects


def _suffixer(name, used_names):
    suffixed_names = (name + '_renamed{0}'.format(i) for i in itertools.count())
    new_names = itertools.chain([name], suffixed_names)
    return next(s for s in new_names if s not in used_names)


class _AttrRefTransformer(ast.NodeTransformer):

    def __init__(self, data):
        self.data = data
        self.data.objlist = []
        self.myhdl_types = (EnumType, SignalType)
        self.name_map = {}

    def visit_Attribute(self, node):
        self.generic_visit(node)

        reserved = ('next', 'posedge', 'negedge', 'max', 'min', 'val', 'signed',
                    'verilog_code', 'vhdl_code')
        if node.attr in reserved:
            return node

        # Don't handle subscripts for now.
        if not isinstance(node.value, ast.Name):
            return node

        # Don't handle locals
        if node.value.id not in self.data.symdict:
            return node

        obj = self.data.symdict[node.value.id]
        # Don't handle enums and functions, handle signals as long as it is a new attribute
        if isinstance(obj, (EnumType, FunctionType)):
            return node
        elif isinstance(obj, SignalType):
            if hasattr(SignalType, node.attr):
                return node

        attrobj = getattr(obj, node.attr)

        orig_name = node.value.id + '.' + node.attr
        if orig_name not in self.name_map:
            base_name = node.value.id + '_' + node.attr
            self.name_map[orig_name] = _suffixer(base_name, self.data.symdict)
        new_name = self.name_map[orig_name]
        self.data.symdict[new_name] = attrobj
        self.data.objlist.append(new_name)

        new_node = ast.Name(id=new_name, ctx=node.value.ctx)
        return ast.copy_location(new_node, node)

    def visit_FunctionDef(self, node):
        nodes = _flatten(node.body, node.args)
        for n in nodes:
            self.visit(n)
        return node



from myhdl import (Signal, intbv, always_comb, conversion, toVHDL, toVerilog)
from myhdl._Simulation import Simulation
from myhdl._traceSignals import traceSignals

import pytest


def bin2gray_depr(B, G, width):

    """ Gray encoder.

    B -- input intbv signal, binary encoded
    G -- output intbv signal, gray encoded
    width -- bit width

    """

    @always_comb
    def comb():
        Bext = intbv(0)[width + 1:]
        Bext[:] = B
        for i in range(width):
            G.next[i] = Bext[i + 1] ^ Bext[i]

    return comb


width = 1
BB = Signal(intbv(0)[width:])
GG = Signal(intbv(0)[width:])


def testOldVerify():
    with pytest.deprecated_call():
        conversion.verify(bin2gray_depr, width, BB, GG)


def testOldAnalyze():
    with pytest.deprecated_call():
        conversion.analyze(bin2gray_depr, width, BB, GG)


def testOldToVHDL():
    with pytest.deprecated_call():
        toVHDL(bin2gray_depr, width, BB, GG)


def testOldToVerilog():
    with pytest.deprecated_call():
        toVerilog(bin2gray_depr, width, BB, GG)


def testOldToTraceSignals():
    with pytest.deprecated_call():
        vcd = traceSignals(bin2gray_depr, width, BB, GG)
        sim = Simulation(vcd)
        sim.run(20)




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2012 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Module with the always_seq decorator. """
from types import FunctionType

from myhdl import AlwaysError, intbv
from myhdl._util import _isGenFunc
from myhdl._Signal import _Signal, _WaiterList, _isListOfSigs
from myhdl._always import _Always, _get_sigdict
from myhdl._instance import _getCallInfo

# evacuate this later
AlwaysSeqError = AlwaysError


class _error:
    pass
_error.EdgeType = "first argument should be an edge"
_error.ResetType = "reset argument should be a ResetSignal"
_error.ArgType = "decorated object should be a classic (non-generator) function"
_error.NrOfArgs = "decorated function should not have arguments"
_error.SigAugAssign = "signal assignment does not support augmented assignment"
_error.EmbeddedFunction = "embedded functions in always_seq function not supported"


class ResetSignal(_Signal):

    def __init__(self, val, active, isasync):
        """ Construct a ResetSignal.

        This is to be used in conjunction with the always_seq decorator,
        as the reset argument.
        """
        _Signal.__init__(self, bool(val))
        self.active = bool(active)
        self.isasync = isasync


def always_seq(edge, reset):
    callinfo = _getCallInfo()
    sigargs = []
    if not isinstance(edge, _WaiterList):
        raise AlwaysSeqError(_error.EdgeType)
    edge.sig._read = True
    edge.sig._used = True
    sigargs.append(edge.sig)
    if reset is not None:
        if not isinstance(reset, ResetSignal):
            raise AlwaysSeqError(_error.ResetType)
        reset._read = True
        reset._used = True
        sigargs.append(reset)
    sigdict = _get_sigdict(sigargs, callinfo.symdict)

    def _always_seq_decorator(func):
        if not isinstance(func, FunctionType):
            raise AlwaysSeqError(_error.ArgType)
        if _isGenFunc(func):
            raise AlwaysSeqError(_error.ArgType)
        if func.__code__.co_argcount > 0:
            raise AlwaysSeqError(_error.NrOfArgs)
        return _AlwaysSeq(func, edge, reset, callinfo=callinfo, sigdict=sigdict)
    return _always_seq_decorator


class _AlwaysSeq(_Always):

    def __init__(self, func, edge, reset, callinfo, sigdict):
        senslist = [edge]
        self.reset = reset
        if reset is not None:
            self.genfunc = self.genfunc_reset
            active = self.reset.active
            isasync = self.reset.isasync
            if isasync:
                if active:
                    senslist.append(reset.posedge)
                else:
                    senslist.append(reset.negedge)
        else:
            self.genfunc = self.genfunc_no_reset

        super(_AlwaysSeq, self).__init__(
            func, senslist, callinfo=callinfo, sigdict=sigdict)

        if self.inouts:
            raise AlwaysSeqError(_error.SigAugAssign, self.inouts)

        if self.embedded_func:
            raise AlwaysSeqError(_error.EmbeddedFunction)

        sigregs = self.sigregs = []
        varregs = self.varregs = []
        for n in self.outputs:
            reg = self.symdict[n]
            if isinstance(reg, _Signal):
                sigregs.append(reg)
            elif isinstance(reg, intbv):
                varregs.append((n, reg, int(reg)))
            else:
                assert _isListOfSigs(reg)
                for e in reg:
                    sigregs.append(e)

    def reset_sigs(self):
        for s in self.sigregs:
            s.next = s._init

    def reset_vars(self):
        for v in self.varregs:
            # only intbv's for now
            _, reg, init = v
            reg._val = init

    def genfunc_reset(self):
        senslist = self.senslist
        if len(senslist) == 1:
            senslist = senslist[0]
        reset_sigs = self.reset_sigs
        reset_vars = self.reset_vars
        func = self.func
        while 1:
            yield senslist
            if self.reset == self.reset.active:
                reset_sigs()
                reset_vars()
            else:
                func()

    def genfunc_no_reset(self):
        senslist = self.senslist
        assert len(senslist) == 1
        senslist = senslist[0]
        func = self.func
        while 1:
            yield senslist
            func()



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Run the unit tests for always_comb """
from random import seed, shuffle
seed('We want repeatable randomness')  # random, but deterministic

import unittest
from unittest import TestCase

from myhdl import (block, Signal, intbv, delay, always_comb, StopSimulation,)
from myhdl._Simulation import Simulation

from .util import setupCosimulation

QUIET = 1


@block
def design1(a, b, c, d, p, q, r):

    def comb():
        p.next = a | b

    return always_comb(comb)


@block
def design2(a, b, c, d, p, q, r):

    def comb():
        p.next = a | b
        q.next = c & d
        r.next = a ^ c

    return always_comb(comb)


@block
def design3(a, b, c, d, p, q, r):

    def comb():
        if a:
            p.next = c | b
            q.next = c & d
            r.next = d ^ c

    return always_comb(comb)


@block
def design4(a, b, c, d, p, q, r):

    def comb():
        p.next = a | b
        q.next = c & d
        r.next = a ^ c
        q.next = c | d

    return always_comb(comb)


@block
def design5(a, b, c, d, p, q, r):

    def comb():
        p.next = a | b
        q.next = c & d
        r.next = a ^ c
        q.next[0] = c | d

    return always_comb(comb)


@block
def design_v(name, a, b, c, d, p, q, r):
    return setupCosimulation(**locals())


class AlwaysCombSimulationTest(TestCase):

    def bench(self, design):

        clk = Signal(0)
        a = Signal(bool(0))
        b = Signal(bool(0))
        c = Signal(bool(0))
        d = Signal(bool(0))
        k = Signal(intbv(0)[8:])
        p = Signal(bool(0))
        q = Signal(intbv(0)[8:])
        r = Signal(bool(0))
        p_v = Signal(bool(0))
        q_v = Signal(intbv(0)[8:])
        r_v = Signal(bool(0))
        vectors = [intbv(j) for __ in range(50) for j in range(16)]
        shuffle(vectors)

        design_inst = design(a, b, c, d, p, q, r).convert(hdl='Verilog')
        design_v_inst = design_v(design.__name__, a, b, c, d, p_v, q_v, r_v)

        def clkGen():
            while 1:
                yield delay(10)
                clk.next ^= 1

        def stimulus():
            for v in vectors:
                a.next = v[0]
                b.next = v[1]
                c.next = v[2]
                d.next = v[3]
                k.next = v
                yield clk.posedge
                yield clk.negedge
                # print p, q, r
                self.assertEqual(p, p_v)
                self.assertEqual(q, q_v)
                self.assertEqual(r, r_v)

            raise StopSimulation("always_comb simulation test")

        return design_inst, design_v_inst, clkGen(), stimulus()

    def test1(self):
        Simulation(self.bench(design1)).run(quiet=QUIET)

    def test2(self):
        Simulation(self.bench(design2)).run(quiet=QUIET)

    def test3(self):
        Simulation(self.bench(design3)).run(quiet=QUIET)

    def test4(self):
        Simulation(self.bench(design4)).run(quiet=QUIET)

    def test5(self):
        Simulation(self.bench(design5)).run(quiet=QUIET)


if __name__ == "__main__":
    unittest.main()



from myhdl import (block, Signal, enum, always_comb)

# t_state = enum('WAIT_POSEDGE', 'WAIT_NEGEDGE', encoding='one_hot')
t_state = enum('WAIT_POSEDGE', 'WAIT_NEGEDGE')


@block
def pcie_legacyint_next_state_logic(state_i, next_state_o, next_state_en_o, interrupt_pending_i, interrupt_assert_o):

        @always_comb
        def sm_output():  # state machine
                if state_i == t_state.WAIT_POSEDGE:
                        interrupt_assert_o.next = 0
                        next_state_en_o   .next = interrupt_pending_i
                        next_state_o      .next = t_state.WAIT_NEGEDGE
                elif state_i == t_state.WAIT_NEGEDGE:
                        interrupt_assert_o.next = 1
                        next_state_en_o   .next = not interrupt_pending_i
                        next_state_o      .next = t_state.WAIT_POSEDGE
                else:
                        interrupt_assert_o.next = 0
                        next_state_en_o   .next = 1
                        next_state_o      .next = t_state.WAIT_POSEDGE

        return sm_output


state = Signal(t_state.WAIT_POSEDGE)
next_state = Signal(t_state.WAIT_POSEDGE)
next_state_en = Signal(bool(0))  # Enable transition to next state
interrupt_pending = Signal(bool(0))
interrupt_assert = Signal(bool(0))


def test_bug_enum_toVHDL():
    pcie_legacyint_next_state_logic(state, next_state, next_state_en, interrupt_pending, interrupt_assert).convert(hdl='VHDL')



#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2015 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" 
    Module that provides the ShadowSignal classes
"""

import warnings
from copy import deepcopy

from myhdl._Signal import _Signal
from myhdl._Waiter import _SignalWaiter, _SignalTupleWaiter
from myhdl._intbv import intbv
from myhdl._simulator import _siglist
from myhdl._bin import bin

# shadow signals


class _ShadowSignal(_Signal):

    __slots__ = ('_waiter',)

    def __init__(self, val):
        _Signal.__init__(self, val)
        # self._driven = True # set this in conversion analyzer

    # remove next attribute assignment
    @_Signal.next.setter
    def next(self, val):
        raise AttributeError("ShadowSignals are readonly")


class _SliceSignal(_ShadowSignal):

    __slots__ = ('_sig', '_left', '_right')

    def __init__(self, sig, left, right=None):
        # XXX error checks
        if right is None:
            _ShadowSignal.__init__(self, sig[left])
        else:
            _ShadowSignal.__init__(self, sig[left:right])
        self._sig = sig
        sig._read = True
        self._left = left
        self._right = right
        if right is None:
            gen = self._genfuncIndex()
        else:
            gen = self._genfuncSlice()
        self._waiter = _SignalWaiter(gen)

    def __repr__(self):
        if self._right is None:
            return repr(self._sig) + '({})'.format(self._left)
        else:
            return repr(self._sig) + '({}, {})'.format(self._left, self._right)

    def _genfuncIndex(self):
        sig, index = self._sig, self._left
        set_next = _Signal.next.fset
        while 1:
            set_next(self, sig[index])
            yield sig

    def _genfuncSlice(self):
        sig, left, right = self._sig, self._left, self._right
        set_next = _Signal.next.fset
        while 1:
            set_next(self, sig[left:right])
            yield sig

    def _setName(self, hdl):
        # if we depend on a ShadowSignal ourselves
        # it would be nice if we 'resolve' the slicing chain
        # e.g. s[7:4][1:0][1] to s[5]
        if isinstance(self._sig, _ShadowSignal):
            # self._sig must have a _left and a _right
            if self._right is None:
                # we're a final index
                self._left = self._sig._right + self._left
            else:
                self._right = self._sig._right + self._right
                self._left = self._sig._right + self._left
            # step one back up
            self._sig = self._sig._sig

        else:
            # we're at the top of the chain
            pass

        if self._right is None:
            if hdl == 'Verilog':
                self._name = "%s[%s]" % (self._sig._name, self._left)
            else:
                self._name = "%s(%s)" % (self._sig._name, self._left)
        else:
            if hdl == 'Verilog':
                self._name = "%s[%s-1:%s]" % (self._sig._name, self._left, self._right)
            else:
                self._name = "%s(%s-1 downto %s)" % (self._sig._name, self._left, self._right)
        # we may have 'shadowed' as well (and further down ...)
        if self._slicesigs:
            for sl in self._slicesigs:
                sl._setName(hdl)

    def _markRead(self):
        self._read = True
        self._sig._read = True

    def _markUsed(self):
        self._used = True
        self._sig._used = True

    def toVerilog(self):
        if self._right is None:
            return "assign %s = %s[%s];" % (self._name, self._sig._name, self._left)
        else:
            return "assign %s = %s[%s-1:%s];" % (self._name, self._sig._name, self._left, self._right)

    def toVHDL(self):
        if self._right is None:
            return "%s <= %s(%s);" % (self._name, self._sig._name, self._left)
        else:
            return "%s <= %s(%s-1 downto %s);" % (self._name, self._sig._name, self._left, self._right)


class ConcatSignal(_ShadowSignal):

    __slots__ = ('_args', '_sigargs', '_initval')

    def __init__(self, *args):
        assert len(args) >= 2
        self._args = args
        self._sigargs = sigargs = []

        nrbits = 0
        val = 0
        for a in args:
            if isinstance(a, intbv):
                w = a._nrbits
                v = a._val
            elif isinstance(a, _Signal):
                sigargs.append(a)
                w = a._nrbits
                if isinstance(a._val, intbv):
                    v = a._val._val
                else:
                    v = a._val
            elif isinstance(a, bool):
                w = 1
                v = a
            elif isinstance(a, str):
                w = len(a)
                v = int(a, 2)
            else:
                raise TypeError("ConcatSignal: inappropriate argument type: %s"
                                % type(a))
            nrbits += w
            val = val << w | v & (1 << w) - 1
        self._initval = val
        ini = intbv(val)[nrbits:]
        _ShadowSignal.__init__(self, ini)
        gen = self.genfunc()
        self._waiter = _SignalTupleWaiter(gen)

    def genfunc(self):
        set_next = _Signal.next.fset
        args = self._args
        sigargs = self._sigargs
        nrbits = self._nrbits
        newval = intbv(self._initval)[nrbits:]
        while 1:
            hi = nrbits
            for a in args:
                if isinstance(a, bool):
                    w = 1
                else:
                    w = len(a)
                lo = hi - w
                # note: 'a in sigargs' is equivalence check, not identity
                if isinstance(a, _Signal):
                    if isinstance(a._val, intbv):
                        newval[hi:lo] = a[w:]
                    else:
                        newval[hi:lo] = a
                hi = lo
            set_next(self, newval)
            yield sigargs

    def _markRead(self):
        self._read = True
        for s in self._sigargs:
            s._markRead()

    def _markUsed(self):
        self._used = True
        for s in self._sigargs:
            s._markUsed()

    def toVHDL(self):
        lines = []
        ini = intbv(self._initval)[self._nrbits:]
        hi = self._nrbits
        for a in self._args:
            if isinstance(a, bool):
                w = 1
            else:
                w = len(a)
            lo = hi - w

            if isinstance(a, _Signal) and a._name is None:
                # We have seen a bug when a concat signal is created in the
                # following way:
                #
                #     sig_list = [Signal(False) for n in range(32)]
                #     concat_sig = ConcatSignal(*reversed(sig_list))
                #
                # It seems that the _name attribute on the signals in sig_list
                # is only updated if an assignment is made to them. Otherwise
                # _name is left as None. We need to check for None and raise
                # a warning.
                from myhdl.conversion._misc import _error
                from myhdl import ToVHDLWarning

                if w == 1:
                    warnings.warn(
                        "%s: %s[%s]" % (_error.UndrivenSignal, self._name, lo),
                        category=ToVHDLWarning)
                else:
                    warnings.warn(
                        "%s: %s[%s:%s]" % (_error.UndrivenSignal, self._name, hi, lo),
                        category=ToVHDLWarning)

            if w == 1:
                if isinstance(a, _Signal) and a._name is not None:
                    # Check that a._name is not None as None should not be
                    # written into the converted code. If it is None then we
                    # assume no assignment has been made to the signal
                    # (otherwise the _name attribute would have been updated
                    # by the _analyzeSigs function). In this situation the
                    # signal should hold its init value (as handled in the
                    # else branch).
                    if a._type == bool:  # isinstance(a._type , bool): <- doesn't work
                        lines.append("%s(%s) <= %s;" % (self._name, lo, a._name))
                    else:
                        lines.append("%s(%s) <= %s(0);" % (self._name, lo, a._name))
                else:
                    lines.append("%s(%s) <= '%s';" % (self._name, lo, bin(ini[lo])))
            else:
                if isinstance(a, _Signal) and a._name is not None:
                    # Check that a._name is not None as None should not be
                    # written into the converted code
                    lines.append("%s(%s-1 downto %s) <= %s;" % (self._name, hi, lo, a._name))
                else:
                    lines.append('%s(%s-1 downto %s) <= "%s";' %
                                 (self._name, hi, lo, bin(ini[hi:lo], w)))
            hi = lo
        return "\n".join(lines)

    def toVerilog(self):
        lines = []
        ini = intbv(self._initval)[self._nrbits:]
        hi = self._nrbits
        for a in self._args:
            if isinstance(a, bool):
                w = 1
            else:
                w = len(a)
            lo = hi - w

            if isinstance(a, _Signal) and a._name is None:
                # We have seen a bug when a concat signal is created in the
                # following way:
                #
                #     sig_list = [Signal(False) for n in range(32)]
                #     concat_sig = ConcatSignal(*reversed(sig_list))
                #
                # It seems that the _name attribute on the signals in sig_list
                # is only updated if an assignment is made to them. Otherwise
                # _name is left as None. We need to check for None and raise
                # a warning.
                from myhdl.conversion._misc import _error
                from myhdl import ToVerilogWarning

                if w == 1:
                    warnings.warn(
                        "%s: %s[%s]" % (_error.UndrivenSignal, self._name, lo),
                        category=ToVerilogWarning)
                else:
                    warnings.warn(
                        "%s: %s[%s:%s]" % (_error.UndrivenSignal, self._name, hi, lo),
                        category=ToVerilogWarning)

            if w == 1:
                if isinstance(a, _Signal) and a._name is not None:
                    # Check that a._name is not None as None should not be
                    # written into the converted code. If it is None then we
                    # assume no assignment has been made to the signal
                    # (otherwise the _name attribute would have been updated
                    # by the _analyzeSigs function). In this situation the
                    # signal should hold its init value (as handled in the
                    # else branch).
                    if a._type == bool:
                        lines.append("assign %s[%s] = %s;" % (self._name, lo, a._name))
                    else:
                        lines.append("assign %s[%s] = %s[0];" % (self._name, lo, a._name))
                else:
                    lines.append("assign %s[%s] = 'b%s;" % (self._name, lo, bin(ini[lo])))
            else:
                if isinstance(a, _Signal) and a._name is not None:
                    # Check that a._name is not None as None should not be
                    # written into the converted code
                    lines.append("assign %s[%s-1:%s] = %s;" % (self._name, hi, lo, a._name))
                else:
                    lines.append("assign %s[%s-1:%s] = 'b%s;" %
                                 (self._name, hi, lo, bin(ini[hi:lo], w)))
            hi = lo
        return "\n".join(lines)

# Tristate signal


class BusContentionWarning(UserWarning):
    pass


warnings.filterwarnings('always', r".*", BusContentionWarning)

# def Tristate(val, delay=None):
#     """ Return a new Tristate(default or delay 0) or DelayedTristate """
#     if delay is not None:
#         if delay < 0:
#             raise TypeError("Signal: delay should be >= 0")
#         return _DelayedTristate(val, delay)
#     else:
#         return _Tristate(val)


def TristateSignal(val):
    return _TristateSignal(val)


class _TristateSignal(_ShadowSignal):

    __slots__ = ('_drivers', '_orival')

    def __init__(self, val):
        self._drivers = []
        # construct normally to set type / size info right
        _ShadowSignal.__init__(self, val)
        self._orival = deepcopy(val)  # keep for drivers
        # reset signal values to None
        self._next = self._val = self._init = None
        self._waiter = _SignalTupleWaiter(self._resolve())

    def driver(self):
        d = _TristateDriver(self)
        self._drivers.append(d)
        return d

    def _resolve(self):
        # set_next = _ShadowSignal._set_next
        senslist = self._drivers
        while 1:
            yield senslist
            res = None
            for d in senslist:
                if res is None:
                    res = d._val
                elif d._val is not None:
                    warnings.warn("Bus contention", category=BusContentionWarning)
                    res = None
                    break
            self._next = res
            _siglist.append(self)

    def toVerilog(self):
        lines = []
        for d in self._drivers:
            if d._driven:
                lines.append("assign %s = %s;" % (self._name, d._name))
        return "\n".join(lines)

    def toVHDL(self):
        lines = []
        for d in self._drivers:
            if d._driven:
                lines.append("%s <= %s;" % (self._name, d._name))
        return "\n".join(lines)

    def __repr__(self):
        return "_TristateSignal(" + repr(self._val) + ")"


class _TristateDriver(_Signal):

    __slots__ = ('_sig',)

    def __init__(self, sig):
        _Signal.__init__(self, sig._orival)
        # reset signal values to None
        self._next = self._val = self._init = None
        self._sig = sig

    @_Signal.next.setter
    def next(self, val):
        if isinstance(val, _Signal):
            val = val._val
        if val is None:
            self._next = None
        else:
            # restore original value to cater for intbv handler
            self._next = self._sig._orival
            self._setNextVal(val)
        _siglist.append(self)

    def __repr__(self):
        return "_TristateDriver(" + repr(self._val) + ")"



'''
Created on 4 feb. 2024

@author: josy
'''

from myhdl import (block, Signal, intbv, delay, always_comb, always_seq, enum,
                   always, instance, StopSimulation, ResetSignal, Constant,
                   conversion)
from myhdl import ConversionError
from myhdl.conversion._misc import _error


@block
def listofenums(Clk, Reset, Start, Progress, Ended):
    listofenums_states = enum('IDLE', 'Doing_Nothing', 'Procrastinating', 'Getting_Ready', 'Executing', encoding='one_hot')

    smn, smp = [Signal(listofenums_states.IDLE) for __ in range(2)]
    stack = [Constant(listofenums_states.IDLE),
             Constant(listofenums_states.Doing_Nothing),
             Constant(listofenums_states.Procrastinating),
             Constant(listofenums_states.Getting_Ready),
             Constant(listofenums_states.Executing)]
    NBR_STATES = len(stack)
    stateindex = Signal(intbv(0)[3:])

    @always_comb
    def smcomb():
        Ended.next = 0

        # if smp == listofenums_states.IDLE:
        if Start:
                smn.next = stack[1]
            # else:
            #     smn.next = smp
        elif Progress:
            if stateindex < NBR_STATES - 1:
                smn.next = stack[stateindex + 1]
            else:
                smn.next = smp
        else:
            smn.next = smp

        '''
            note that if we exchange the two following lines
            we get a failing conversion to VHDL (only; Verilog is fine ...) 
        '''
        if smp == listofenums_states.Executing:
        # if stateindex == NBR_STATES - 1:
            Ended.next = 1

        # smn.next = smp
        # if smp == listofenums_states.IDLE:
        #     if Start:
        #         smn.next = listofenums_states.Doing_Nothing
        #
        # elif smp == listofenums_states.Doing_Nothing:
        #     if Start:
        #         smn.next = listofenums_states.Procrastinating
        #
        # elif smp == listofenums_states.Procrastinating:
        #     if Start:
        #         smn.next = listofenums_states.Getting_Ready
        #
        # elif smp == listofenums_states.Getting_Ready:
        #     if Start:
        #         smn.next = listofenums_states.Executing
        #
        # elif smp == listofenums_states.Executing:
        #     smn.next = listofenums_states.Executing
        #     Ended.next = 1
        #
        # else:
        #     smn.next = listofenums_states.IDLE

    @always_seq(Clk.posedge, reset=Reset)
    def smsync():
        smp.next = smn
        if Start or Progress:
            if stateindex < NBR_STATES - 1:
                stateindex.next = stateindex + 1

    return smcomb, smsync


def test_listofenums():
    Clk, Start, Progress, Ended = [Signal(bool(0)) for __ in range(4)]
    Reset = ResetSignal(0, 1, False)
    assert listofenums(Clk, Reset, Start, Progress, Ended).analyze_convert() == 0


if __name__ == '__main__':
    Clk, Start, Progress, Ended = [Signal(bool(0)) for __ in range(4)]
    Reset = ResetSignal(0, 1, False)
    dfc = listofenums(Clk, Reset, Start, Progress, Ended)
    dfc.convert(hdl='Verilog', initial_values=True)
    dfc.convert(hdl='VHDL', initial_values=True)



import myhdl
from myhdl import *

from bin2gray2 import bin2gray
from inc import Inc

def GrayInc(graycnt, enable, clock, reset, width):
    
    bincnt = Signal(modbv(0)[width:])
    
    inc_1 = Inc(bincnt, enable, clock, reset)
    bin2gray_1 = bin2gray(B=bincnt, G=graycnt, width=width)
    
    return inc_1, bin2gray_1


def GrayIncReg(graycnt, enable, clock, reset, width):
    
    graycnt_comb = Signal(modbv(0)[width:])
    
    gray_inc_1 = GrayInc(graycnt_comb, enable, clock, reset, width)

    @always(clock.posedge)
    def reg_1():
        graycnt.next = graycnt_comb
    
    return gray_inc_1, reg_1


def main():
    width = 8
    graycnt = Signal(modbv(0)[width:])
    enable = Signal(bool())
    clock = Signal(bool())
    reset = ResetSignal(0, active=0, isasync=True)

    toVerilog(GrayIncReg, graycnt, enable, clock, reset, width)
    toVHDL(GrayIncReg, graycnt, enable, clock, reset, width)

          
if __name__ == '__main__':
    main()


            
            

    

    
        


                

        


  



from myhdl import (block, Signal, intbv, always_comb, concat)


@block
def bug_43(sigin, sigout):

    @always_comb
    def comb():
        # This does not generate correct VHDL code (resize is missing)
        sigout.next = concat(sigin[0], sigin[2])

        # The following does work:
        tmp = concat(sigin[0], sigin[2])
        sigout.next = tmp

    return comb


def test_bug_43():
    sigin = Signal(intbv(0)[4:])
    sigout = Signal(intbv(0)[4:])

    assert bug_43(sigin, sigout).analyze_convert() == 0




'''
Created on 3 dec. 2024

@author: josy
'''

from myhdl import (HdlClass, Signal, intbv, block, always_seq, instances, OpenPort, always_comb,
                   Constant)


class Counter(HdlClass):

    def __init__(self, RANGE, Clk, Reset, SClr, CntEn, Q=None, IsMax=None, WRAP_AROUND=False):
        '''
            RANGE: int : number of counts
            Clk: Signal(bool()): the domain clock
            Reset: ResetSignal(): its associated reset
            SClr: Signal(bool()): resets the count
            CntEn: Signal(bool()): advances count
            Q: Signal(intbv()[w:]): the actual count
        '''
        self.Clk = Clk
        self.Reset = Reset
        self.SClr = SClr
        self.CntEn = CntEn
        self.RANGE = RANGE
        self.Q = Q if Q is not None else Signal(intbv(0, 0, RANGE))
        self.IsMax = IsMax if IsMax is not None else Signal(bool(0))
        self.WRAP_AROUND = WRAP_AROUND

    @block(skipname=True)
    def hdl(self):
        if isinstance(self.Q, OpenPort):
            count = Signal(intbv(0, 0, self.RANGE))
        else:
            count = self.Q.duplicate()

        if self.WRAP_AROUND:

            @always_seq(self.Clk.posedge, reset=self.Reset)
            def upcounter():
                if self.SClr or self.CntEn:
                    if self.SClr:
                        count.next = 0
                    else:
                        if count < (self.RANGE - 1):
                            count.next = count + 1
                        else:
                            count.next = 0

        else:

            @always_seq(self.Clk.posedge, reset=self.Reset)
            def upcounter():
                if self.SClr or self.CntEn:
                    if self.SClr:
                        count.next = 0
                    else:
                        if count < (self.RANGE - 1):
                            count.next = count + 1

        @always_comb
        def mkq():
            self.Q.next = count
            self.IsMax.next = (count == self.RANGE - 1)

        return self.hdlinstances()


class Tone(HdlClass):

    def __init__(self, DIVIDER, Clk, Reset, Key, Wave=None):
        self.DIVIDER = DIVIDER
        self.Clk = Clk
        self.Reset = Reset
        self.Key = Key
        self.Wave = Wave if Wave is not None else Signal(bool())

    @block(skipname=True)
    def hdl(self):
        divider = Counter(self.DIVIDER, self.Clk, self.Reset, Constant(bool(0)), Constant(bool(1)),
                          Q=OpenPort(), WRAP_AROUND=True)

        @always_seq(self.Clk.posedge, reset=self.Reset)
        def squarewave():
            if not self.Key:
                self.Wave.next = 0
            else:
                if divider.IsMax:
                    self.Wave.next = not self.Wave

        return self.hdlinstances()


if __name__ == '__main__':

    from myhdl import ResetSignal

    class KakaFonie(HdlClass):

        def __init__(self, Clk, Reset, Keys, Noise):
            self.Clk = Clk
            self.Reset = Reset
            self.Keys = Keys
            self.Noise = Noise if Noise is not None else Signal(bool(0))

        @block(skipname=True)
        def hdl(self):
            FREQUENCIES = [261.626, 277.183, 293.665, 311.127, 329.628, 349.228, 369.994, 391.995, 415.305, 440.0, 466.167, 493.883]

            wavegenerator = []
            # waves = [Signal(bool(0)) for __ in range(12)]
            for i in range(12):
                DIVIDER = int(50e6 / FREQUENCIES[i])
                # wavegenerator.append(Tone(DIVIDER, self.Clk, self.Reset, self.Keys(1), waves[i]))
                wavegenerator.append(Tone(DIVIDER, self.Clk, self.Reset, self.Keys(i)))

            waves = [wavegenerator[i].Wave for i in range(12)]

            @always_comb
            def makenoise():
                self.Noise.next = 0
                for i in range(12):
                    if waves[i]:
                        self.Noise.next = self.Noise | 1

            return self.hdlinstances()

    # try converting
    Clk = Signal(bool(0))
    Reset = ResetSignal(0, 1, False)
    Keys = Signal(intbv(0)[12:])
    Noise = Signal(bool(0))

    if 1:
        ''' looks like we have to live with writing a wrapper '''

        # a local written-out wrapper works fine
        @block
        def wrapper(Clk, Reset, Keys, Noise):
            dfc = KakaFonie(Clk, Reset, Keys, Noise)
            # print(f'{vars(dfc)=}')
            dfchdl = dfc.hdl()
            # print(f'{vars(dfchdl)=}')
            return dfchdl

        dfc = wrapper(Clk, Reset, Keys, Noise)
        dfc.convert(hdl='VHDL', name='KakaFonie')
        dfc.convert(hdl='Verilog', name='KakaFonie')
    # these fail in one way or another
    # else:
    #     from _hdlclass import wrapper, convert
    #     if 1:
    #         # this raises an IndexError in _analyze.py
    #         # beacuse the '*args' in wrapper disappear into nowhere?
    #         convert(wrapper(XYMotors, PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive))
    #     else:
    #         # this produces an 'empty' entity
    #         hc = XYMotors(PWMCOUNT, Clk, Reset, XSpeed, YSpeed, XDrive, YDrive)
    #         hc.convert(hdl='VHDL', name='XYMotors')




#  This file is part of the myhdl library, a Python package for using
#  Python as a Hardware Description Language.
#
#  Copyright (C) 2003-2008 Jan Decaluwe
#
#  The myhdl library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License as
#  published by the Free Software Foundation; either version 2.1 of the
#  License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.

#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

""" Module with the always function. """
from types import FunctionType

from myhdl import AlwaysError
from myhdl._util import _isGenFunc
from myhdl._delay import delay
from myhdl._Signal import _Signal
from myhdl._Signal import _WaiterList
from myhdl._Waiter import _Waiter, _SignalWaiter, _SignalTupleWaiter, \
    _DelayWaiter, _EdgeWaiter, _EdgeTupleWaiter
from myhdl._instance import _Instantiator, _getCallInfo


class _error:
    pass
_error.DecArgType = "decorator argument should be a Signal, edge, or delay"
_error.ArgType = "decorated object should be a classic (non-generator) function"
_error.NrOfArgs = "decorated function should not have arguments"
_error.DecNrOfArgs = "decorator should have arguments"


def _get_sigdict(sigs, symdict):
    """Lookup signals in caller namespace and return sigdict

    Lookup signals in then namespace of a caller. This is used to add
    signal arguments from an instantiator decorator to the instance.
    0: this function
    1: the instantiator decorator
    2: the module function that defines instances
    """

    sigdict = {}
    for n, v in symdict.items():
        for s in sigs:
            if s is v:
                sigdict[n] = s
    return sigdict


def always(*args):
    callinfo = _getCallInfo()
    sigargs = []
    for arg in args:
        if isinstance(arg, _Signal):
            arg._read = True
            arg._used = True
            sigargs.append(arg)
        elif isinstance(arg, _WaiterList):
            arg.sig._read = True
            arg.sig._used = True
            sigargs.append(arg.sig)
        elif not isinstance(arg, delay):
            raise AlwaysError(_error.DecArgType)
    sigdict = _get_sigdict(sigargs, callinfo.symdict)

    def _always_decorator(func):
        if not isinstance(func, FunctionType):
            raise AlwaysError(_error.ArgType)
        if _isGenFunc(func):
            raise AlwaysError(_error.ArgType)
        if func.__code__.co_argcount > 0:
            raise AlwaysError(_error.NrOfArgs)
        return _Always(func, args, callinfo=callinfo, sigdict=sigdict)
    return _always_decorator


class _Always(_Instantiator):

    def __init__(self, func, senslist, callinfo, sigdict=None):
        self.func = func
        self.senslist = tuple(senslist)
        super(_Always, self).__init__(self.genfunc, callinfo=callinfo)
        # update sigdict with decorator signal arguments
        if sigdict is not None:
            self.sigdict.update(sigdict)

    @property
    def funcobj(self):
        return self.func

    def _waiter(self):
        # infer appropriate waiter class
        # first infer base type of arguments
        for t in (_Signal, _WaiterList, delay):
            if isinstance(self.senslist[0], t):
                bt = t
        for s in self.senslist[1:]:
            if not isinstance(s, bt):
                bt = None
                break
        # now set waiter class
        w = _Waiter
        if bt is delay:
            w = _DelayWaiter
        elif len(self.senslist) == 1:
            if bt is _Signal:
                w = _SignalWaiter
            elif bt is _WaiterList:
                w = _EdgeWaiter
        else:
            if bt is _Signal:
                w = _SignalTupleWaiter
            elif bt is _WaiterList:
                w = _EdgeTupleWaiter
        return w

    def genfunc(self):
        senslist = self.senslist
        if len(senslist) == 1:
            senslist = senslist[0]
        func = self.func
        while 1:
            yield senslist
            func()



from myhdl import (block, Signal, intbv, delay, always_comb,
                   always, instance, StopSimulation,
                   conversion)
from myhdl import ConversionError
from myhdl.conversion._misc import _error

N = 8
M = 2 ** N

### A first case that already worked with 5.0 list of signal constraints ###


@block
def intbv2list():
    """Conversion between intbv and list of boolean signals."""

    a = Signal(intbv(0)[N:])
    b = [Signal(bool(0)) for i in range(len(a))]
    z = Signal(intbv(0)[N:])

    @always(a)
    def extract():
        for i in range(len(a)):
            b[i].next = a[i]

    @always(*b)
    def assemble():
        for i in range(len(b)):
            z.next[i] = b[i]

    @instance
    def stimulus():
        for i in range(M):
            a.next = i
            yield delay(10)
            assert z == a
            print(a)
        raise StopSimulation

    return extract, assemble, stimulus

# test


def test_intbv2list():
    assert conversion.verify(intbv2list()) == 0

### A number of cases with relaxed constraints, for various decorator types ###


@block
def inv1(z, a):

    @always(a)
    def comb():
        z.next = not a

    return comb


@block
def inv2(z, a):

    @always_comb
    def comb():
        z.next = not a

    return comb


@block
def inv3(z, a):

    @instance
    def comb():
        while True:
            yield a
            z.next = not a

    return comb


@block
def inv4(z, a):

    @instance
    def comb():
        while True:
            yield a
            yield delay(1)
            z.next = not a

    return comb


@block
def case1(z, a, inv):
    b = [Signal(bool(1)) for __ in range(len(a))]
    c = [Signal(bool(0)) for __ in range(len(a))]

    @always(a)
    def extract():
        for i in range(len(a)):
            b[i].next = a[i]

    inst = [None] * len(b)
    for i in range(len(b)):
        inst[i] = inv(c[i], b[i])

    @always(*c)
    def assemble():
        for i in range(len(c)):
            z.next[i] = c[i]

    return extract, inst, assemble


@block
def case2(z, a, inv):
    b = [Signal(bool(1)) for __ in range(len(a))]
    c = [Signal(bool(0)) for __ in range(len(a))]

    @always_comb
    def extract():
        for i in range(len(a)):
            b[i].next = a[i]

    inst = [None] * len(b)
    for i in range(len(b)):
        inst[i] = inv(c[i], b[i])

    @always_comb
    def assemble():
        for i in range(len(c)):
            z.next[i] = c[i]

    return extract, inst, assemble


@block
def case3(z, a, inv):
    b = [Signal(bool(1)) for __ in range(len(a))]
    c = [Signal(bool(0)) for __ in range(len(a))]

    @instance
    def extract():
        while True:
            yield a
            for i in range(len(a)):
                b[i].next = a[i]

    inst = [None] * len(b)
    for i in range(len(b)):
        inst[i] = inv(c[i], b[i])

    @instance
    def assemble():
        while True:
            yield c
            for i in range(len(c)):
                z.next[i] = c[i]

    return extract, inst, assemble


@block
def case4(z, a, inv):
    b = [Signal(bool(1)) for __ in range(len(a))]
    c = [Signal(bool(0)) for __ in range(len(a))]

    @instance
    def extract():
        while True:
            yield a
            yield delay(1)
            for i in range(len(a)):
                b[i].next = a[i]

    inst = [None] * len(b)
    for i in range(len(b)):
        inst[i] = inv(c[i], b[i])

    @instance
    def assemble():
        while True:
            yield c
            yield delay(1)
            for i in range(len(c)):
                z.next[i] = c[i]

    return extract, inst, assemble


@block
def processlist(case, inv):
    """Extract list from intbv, do some processing, reassemble."""

    a = Signal(intbv(1)[N:])
    z = Signal(intbv(0)[N:])

    case_inst = case(z, a, inv)

    @instance
    def stimulus():
        for i in range(M):
            yield delay(10)
            a.next = i
            yield delay(10)
            assert z == ~a
            print(z)
        raise StopSimulation

    return case_inst, stimulus

# functional tests


def test_processlist11():
    assert conversion.verify(processlist(case1, inv1)) == 0


def test_processlist12():
    assert conversion.verify(processlist(case1, inv2)) == 0


def test_processlist22():
    assert conversion.verify(processlist(case2, inv2)) == 0


def test_processlist33():
    assert conversion.verify(processlist(case3, inv3)) == 0


def test_processlist44():
    assert conversion.verify(processlist(case4, inv4)) == 0


# signed and unsigned
@block
def unsigned():
    z = Signal(intbv(0)[8:])
    a = [Signal(intbv(0)[8:]) for __ in range(3)]

    @always_comb
    def comb():
        z.next = a[1] + a[2]

    @instance
    def stimulus():
        a[0].next = 2
        a[1].next = 5
        yield delay(10)
        print(z)

    return comb, stimulus


def test_unsigned():
    conversion.verify(unsigned())


@block
def signed():
    z = Signal(intbv(0, min=-10, max=34))
    a = [Signal(intbv(0, min=-5, max=17)) for __ in range(3)]

    @always_comb
    def comb():
        z.next = a[1] + a[2]

    @instance
    def stimulus():
        a[0].next = 2
        a[1].next = -5
        yield delay(10)
        print(z)

    return comb, stimulus


def test_signed():
    conversion.verify(signed())


@block
def mixed():
    z = Signal(intbv(0, min=0, max=34))
    a = [Signal(intbv(0, min=-11, max=17)) for __ in range(3)]
    b = [Signal(intbv(0)[5:]) for __ in range(3)]

    @always_comb
    def comb():
        z.next = a[1] + b[2]

    @instance
    def stimulus():
        a[0].next = -6
        b[2].next = 15
        yield delay(10)
        print(z)

    return comb, stimulus


def test_mixed():
    conversion.verify(mixed())

# ## error tests

# port in list


@block
def portInList(z, a, b):

    m = [a, b]

    @always_comb
    def comb():
        z.next = m[0] + m[1]

    return comb


def test_portInList():
    z, a, b = [Signal(intbv(0)[8:]) for __ in range(3)]

    try:
        inst = conversion.analyze(portInList(z, a, b))
    except ConversionError as e:
        assert e.kind == _error.PortInList
    else:
        assert False

# signal in multiple lists


@block
def sigInMultipleLists():

    z, a, b = [Signal(intbv(0)[8:]) for __ in range(3)]

    m1 = [a, b]
    m2 = [a, b]

    @always_comb
    def comb():
        z.next = m1[0] + m2[1]

    return comb


def test_sigInMultipleLists():

    try:
        inst = conversion.analyze(sigInMultipleLists())
    except ConversionError as e:
        assert e.kind == _error.SignalInMultipleLists
    else:
        assert False

# list of signals as port


@block
def my_register(clk, inp, outp):

    @always(clk.posedge)
    def my_register_impl():
        for index in range(len(inp)):
            outp[index].next = inp[index]

    return my_register_impl


def test_listAsPort():
    count = 3
    clk = Signal(False)
    inp = [Signal(intbv(0)[8:0]) for __ in range(count)]
    outp = [Signal(intbv(0)[8:0]) for __ in range(count)]
    try:
        inst = conversion.analyze(my_register(clk, inp, outp))
    except ConversionError as e:
        assert e.kind == _error.ListAsPort
    else:
        assert False

